@@ -1054,9 +1054,9 @@ void MEMORY_CartA0bfEnable(void)
 void MEMORY_GetCharset(UBYTE *cs)
 {
 	/* copy font, but change screencode order to ATASCII order */
-	memcpy(cs, emuos_h + 0x200, 0x100); /* control chars */
-	memcpy(cs + 0x100, emuos_h, 0x200); /* !"#$..., uppercase letters */
-	memcpy(cs + 0x300, emuos_h + 0x300, 0x100); /* lowercase letters */
+	memcpy(cs, emuos_h + EMUOS_CHARSET_OFFSET + 0x200, 0x100); /* control chars */
+	memcpy(cs + 0x100, emuos_h + EMUOS_CHARSET_OFFSET, 0x200); /* !"#$..., uppercase letters */
+	memcpy(cs + 0x300, emuos_h + EMUOS_CHARSET_OFFSET + 0x300, 0x100); /* lowercase letters */
 }
 
 #ifndef PAGED_MEM
